/*
** Module   : BasePrimitives.java
** Abstract : Base class for output primitives.
**
** Copyright (c) 2009-2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ----------------------------Description-----------------------------
** 001 SIY 20090911   @43886 Created initial version
** 002 SIY 20100619          Switching to own implementation of character UI.
** 003 SIY 20100804          Changes in imports, some refactorings.
** 004 SIY 20100822          Refactored AttributeContainer support.
** 005 SIY 20100915          locateFrame() is moved to UiUtils.
** 006 SIY 20110112          Changes related to new color handling scheme.
** 007 GES 20140627          Changed color attribute method names to match naming standards.
** 008 CA  20140805          Do not allow ChUI impl classes access to APIs outside of 
**                           ChuiOutputManager.
**                           Moved here ChUI specific primitives from OutputPrimitives. 
** 009 CA  20141119          Refactored drawing code to enable multiple OS windows, for GUI
**                           support.  Misc changes/fixes related to drawing on multiple windows.
*/

package com.goldencode.p2j.ui.chui;

import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.ui.client.driver.*;
import com.goldencode.p2j.ui.client.widget.*;

/**
 * Base class for all other {@link OutputPrimitives} implementations used by
 * CHUI.
 */
public abstract class BasePrimitives
implements OutputPrimitives
{
   /**
    * Empty cell used for implementations which don't support
    * {@link #cellAt(int, int)}
    */
   protected static final Cell EMPTY_CELL = new Cell(' ', Color.DEFAULT);

   /**
    * Draw upper left corner.
    * <p>
    * The driver-level implementation does not need to implement this method 
    * because it is never reaches the driver.
    * 
    * @param   col
    *          Column.
    * @param   row
    *          Row.
    * @param   color
    *          Color.
    */
   public abstract void drawULCorner(int col, int row, Color color);

   /**
    * Draw upper right corner.
    * <p>
    * The driver-level implementation does not need to implement this method 
    * because it is never reaches the driver.
    * 
    * @param   col
    *          Column.
    * @param   row
    *          Row.
    * @param   color
    *          Color.
    */
   public abstract void drawURCorner(int col, int row, Color color);

   /**
    * Draw lower left corner.
    * <p>
    * The driver-level implementation does not need to implement this method 
    * because it is never reaches the driver.
    * 
    * @param   col
    *          Column.
    * @param   row
    *          Row.
    * @param   color
    *          Color.
    */
   public abstract void drawLLCorner(int col, int row, Color color);

   /**
    * Draw lower right corner.
    * <p>
    * The driver-level implementation does not need to implement this method 
    * because it is never reaches the driver.
    * 
    * @param   col
    *          Column.
    * @param   row
    *          Row.
    * @param   color
    *          Color.
    */
   public abstract void drawLRCorner(int col, int row, Color color);

   /**
    * Draw horizontal border.
    * <p>
    * The driver-level implementation does not need to implement this method 
    * because it is never reaches the driver.
    * 
    * @param   col
    *          Column.
    * @param   row
    *          Row.
    * @param   width
    *          Border width. 
    * @param   color
    *          Color.
    */
   public abstract void drawHBorder(int col, int row, int width, Color color);

   /**
    * Draw vertical border.
    * <p>
    * The driver-level implementation does not need to implement this method 
    * because it is never reaches the driver.
    * 
    * @param   col
    *          Column.
    * @param   row
    *          Row.
    * @param   height
    *          Border height. 
    * @param   color
    *          Color.
    */
   public abstract void drawVBorder(int col, int row, int height, Color color);

   /**
    * Draw complete box (with horizontal and vertical lines as well as corners)
    * using pseudo-graphic characters (or replacement characters, if 
    * pseudo-graphic characters are not supported). Code, which uses this method
    * assumes that vertical sides of the box consume exactly one character 
    * position in width and horizontal sides of the box consume exactly one 
    * character position in height. 
    * <p>
    * The driver-level implementation does not need to implement this method 
    * because it is never passed reaches the driver.
    * 
    * @param   left
    *          Left coordinate of the box.
    * @param   top
    *          Top coordinate of the box.
    * @param   right
    *          Right coordinate of the box.
    * @param   bottom
    *          Bottom coordinate of the box.
    * @param   color
    *          Color.
    */
   public abstract void box(int left, int top, int right, int bottom, Color color);
   /**
    * Append char to the screen at current cursor location and move the cursor.
    * <p>
    * The driver-level implementation does not need to implement this method 
    * because it is never reaches the driver.
    * 
    * @param   chr
    *          Character to add.
    * @param   color
    *          Color.
    */
   public abstract void append(int chr, Color color);

   /**
    * Append string to the screen at current cursor location and move the
    * cursor.
    * <p>
    * Particular implementation of this method should perform direct output to 
    * the screen and avoid any buffering. Note that input string is a Unicode 
    * string, so support of the Unicode is up to implementation. Note that 
    * this method is used at the very beginning of the application startup 
    * (usually at the user authentication phase) and remaining interaction 
    * with the user is performed via buffered output which uses bulk screen 
    * data transfer (see {@link #updateScreen(ScreenData)} for more details), 
    * monitors changed areas etc. 
    * 
    * @param   str
    *          String to add.
    * @param   color
    *          Color.
    */
   public abstract void append(String str, Color color);

   /**
    * Get absolute cursor X coordinate.
    * <p>
    * Driver-level {@link OutputPrimitives} implementation should return actual
    * cursor position at physical screen.
    * 
    * @return  cursor X coordinate.
    */
   public abstract int getX();
   
   /**
    * Get absolute cursor Y coordinate.
    * <p>
    * Driver-level {@link OutputPrimitives} implementation should return actual
    * cursor position at physical screen.
    * 
    * @return  cursor Y coordinate.
    */
   public abstract int getY();
   
   /**
    * Position the real cursor.
    * <p>
    * TODO: in CRARVA driver at low level there is no difference between 
    * cursorStay and cursorAt, they both call same native method. Seems 
    * cursorStay can be completely dropped, at least for driver level. 
    * 
    * @param     x
    *            Cursor X coordinate. 
    * @param     y 
    *            Cursor Y coordinate. 
    * 
    * @return    <code>true</code> if cursor positioning succeeded.
    */
   public abstract boolean cursorStay(int x, int y);

   /**
    * Position physical cursor to specified screen location.
    * <p>
    * Low-level implementation of this method (i.e. implementation used by the 
    * particular screen driver) should perform actual physical cursor movement
    * so it will be visible to user if cursor is enabled (see 
    * {@link ScreenDriver#setCursorStatus(boolean)} for more details).  
    * 
    * @param   x
    *          Cursor X.
    * @param   y
    *          Cursor Y.
    * 
    * @return  <code>true</code> if operation was successful.
    */
   public abstract boolean cursorAt(int x, int y);

   /**
    * Draw horizontal line using pseudo-graphic characters. If pseudo-graphic 
    * characters are not supported, some kind of replacement character is used 
    * (for example, some ASCII character).
    * <p>
    * The driver-level implementation does not need to implement this method 
    * because it is never reaches the driver.
    * 
    * @param   length
    *          Line length.
    * @param   color
    *          Color.
    */
   public abstract void hLine(int length, Color color);

   /**
    * Draw vertical line using pseudo-graphic characters. If pseudo-graphic 
    * characters are not supported, some kind of replacement character is used 
    * (for example, some ASCII character).
    * <p>
    * The driver-level implementation does not need to implement this method 
    * because it is never reaches the driver.
    * 
    * @param   length
    *          Line length.
    * @param   color
    *          Color.
    */
   public abstract void vLine(int length, Color color);

   /**
    * Get character at specified position.
    *
    * @param    row
    *           Character row.
    * @param    col
    *           Character column.
    *
    * @return   Cell from specified position or cell consisting of space
    *           and default color if operation is not supported.
    */
   public Cell cellAt(int row, int col)
   {
      return EMPTY_CELL;
   }

   /**
    * Get color for the component area.
    *
    * @param    widget
    *           Widget reference.
    *
    * @return   Array with two elements: attribute and color.
    */
   public Color widgetColor(Widget<?> widget)
   {
      Widget<?> frame = UiUtils.locateFrame(widget);

      return frame != null ? UiUtils.getDColor(frame) : Color.NORMAL;
   }

   /**
    * Get color for border of specified widget.
    *
    * @param    widget
    *           Widget reference.
    *
    * @return   Array with two elements: attribute and color.
    */
   public Color borderColor(Widget<?> widget)
   {
      if (widget.config() == null)
         return Color.DEFAULT;
      
      if (widget instanceof SubMenu)
      {
         return UiUtils.getDColor(widget.<SubMenuConfig>config().dcolor, widget);
      }

      if (!(widget.config() instanceof BaseConfig))
      {
         widget = UiUtils.locateFrame(widget);
      }

      return UiUtils.getDColor(widget);
   }

   /**
    * Check if this output primitives implementation requires border corners
    * when drawing a box. This implementation requires box border corners,
    * so this method will return <code>true</code>.
    *
    * @return   <code>true</code>, as the box will be drawn with border
    *           corners.
    */
   public boolean hasBoxBorderCorners()
   {
      return true;
   }

   /**
    * Perform a bulk output operation for the whole screen at once with
    * output masking when desired. Set the cursor afterwards.
    *
    * @param   screen
    *          Container for whole screen content and cursor position.
    */
   public void updateScreen(ScreenData screen)
   {
      // do nothing
   }
   
   /**
    * Get the ID of the window currently selected.
    * 
    * @return   In ChUI, always the {@link WidgetId#_DEFAULT_WINDOW_ID}.
    */
   @Override
   public int getWindowId()
   {
      return WidgetId._DEFAULT_WINDOW_ID;
   }
   
   /**
    * On a device which allows multiple windows to be created, this allows selecting an explicit
    * window before using the primitives or before drawing a widget.
    * <p>
    * For ChUI, this is a no-op.
    * 
    * @param    windowId
    *           Window ID.
    */
   @Override
   public void selectWindow(int windowId)
   {
      // no-op
   }
}

